# -*- coding: utf-8 -*-
# pylint: disable-msg = C0103, W0201, W0232, W0612, W0613, W0703, E0213
#
# Copyright 2004-2006 André Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.main

@var _tests: The test classes
@type _tests: C{list}
"""
__author__    = "André Malo"
__docformat__ = "epytext en"
__all__       = ['getSuite']

import unittest, svnmailer_test.util
_tests = []


#### Test Bases                                     ####
########################################################
class mockingTestBase(svnmailer_test.util.MockingBase):
    """ Base class for module mocking settings tests """

    def getStructDict(self):
        """ Returns the dict needed for svnmailer.settings initialization """
        def members(space, the_members, aliases = None, typemap = None):
            return []
        class Struct(object):
            _mocked_object = "magic"
            pass
        class MemberDescriptor(object):
            _mocked_object = "magic"
            pass

        return {
            'members': members,
            'Struct' : Struct,
            'MemberDescriptor': MemberDescriptor,
        }

    def getStructAccessorsDict(self):
        """ Returns the dict needed for svnmailer.settings initialization """
        return {
            'typemap': {'mocked': 'typemap'}
        }

    def getInitialMocks(self):
        """ Returns the list initially mocked modules """
        return [
            ('svnmailer.typedstruct', self.getStructDict()),
            ('svnmailer.struct_accessors', self.getStructAccessorsDict()),
        ]

class settingsTestBase(mockingTestBase):
    """ Bases class for settings tests """

    def setUp(self):
        """ set up test environment """
        super(settingsTestBase, self).setUp()

        from svnmailer import settings
        reload(settings) # make sure to initialize again
        self._settings = settings

    def tearDown(self):
        """ tear down test environment """
        del self._settings
        super(settingsTestBase, self).tearDown()

        # leave a clean module around
        from svnmailer import settings
        reload(settings)

class memberTestBase(settingsTestBase):
    """ Defines an alternate members function """
    _members_return = ['a']

    def getStructDict(self):
        """ store the args for later evaluation """
        odict = super(memberTestBase, self).getStructDict()
        def members(space, the_members, aliases = None, typemap = None):
            self._member_args = {
                'space': space,
                'the_members': the_members,
                'aliases': aliases,
                'typemap': typemap,
            }
            return self._members_return

        odict['members'] = members
        return odict

class containerTestBase(settingsTestBase):
    """ Defines an alternate members function """
    _members_return = []

    def getStructDict(self):
        """ store the args for later evaluation """
        odict = super(containerTestBase, self).getStructDict()
        def members(space, the_members, aliases = None, typemap = None):
            space['_member_args'] = {
                'the_members': the_members,
                'aliases': aliases,
                'typemap': typemap,
            }
            return self._members_return

        odict['members'] = members
        return odict


#### Test Cases                                     ####
########################################################
class importTestCase(mockingTestBase):
    """ Importing svnmailer.settings """

    def testImport(self):
        """ from svnmailer import settings """
        try:
            from svnmailer import settings
        except Exception, exc:
            self.fail(
                "svnmailer.settings could not be imported: %s" % str(exc)
            )

_tests.append(importTestCase)


class groupMembersTestCase(memberTestBase):
    """ svnmailer.settings.groupMembers """

    def test00space(self):
        """ groupMembers passes the space around """
        space = {'mocked': 'space'}
        self._settings.groupMembers(space)

        expected = {'mocked': 'space'}
        result = self._member_args['space']
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test01typemap(self):
        """ groupMembers passes struct_accessors.typemap """
        self._settings.groupMembers({})

        expected = {'mocked': 'typemap'}
        result = self._member_args['typemap']
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02members(self):
        """ groupMembers passes the proper member definitions """
        self._settings.groupMembers({})

        expected = {
            '_name'                      : 'unicode',
            '_def_for_repos'             : 'regex',
            '_def_for_paths'             : 'regex',
            'for_repos'                  : ('regex',      {'map': True}),
            'for_paths'                  : ('regex',      {'map': True}),
            'exclude_paths'              : ('regex',      {'map': True}),
            'ignore_if_other_matches'    : 'humanbool',
            'show_nonmatching_paths'     : ('token',
                                           {'map': True, 'allowed': (u'yes',
                                            u'no', u'ignore')}),
            'commit_subject_template'    : ('unicode',    {'map': True}),
            'propchange_subject_template': ('unicode',    {'map': True}),
            'lock_subject_template'      : ('unicode',    {'map': True}),
            'unlock_subject_template'    : ('unicode',    {'map': True}),
            'commit_subject_prefix'      : ('unicode',
                                           {'subst': True, 'map': True}),
            'propchange_subject_prefix'  : ('unicode',
                                           {'subst': True, 'map': True}),
            'lock_subject_prefix'        : ('unicode',
                                           {'subst': True, 'map': True}),
            'unlock_subject_prefix'      : ('unicode',
                                           {'subst': True, 'map': True}),
            'max_subject_length'         : 'int',
            'from_addr'                  : ('tokenlist',
                                           {'subst': True, 'map': True}),
            'to_addr'                    : ('tokenlist',
                                           {'subst': True, 'map': True}),
            'to_fake'                    : ('unicode',
                                           {'subst': True, 'map': True}),
            'bcc_addr'                   : ('tokenlist',
                                           {'subst': True, 'map': True}),
            'reply_to_addr'              : ('unicode',
                                           {'subst': True, 'map': True}),
            'to_newsgroup'               : ('tokenlist',
                                           {'subst': True, 'map': True}),
            'diff_command'               : ('unicommand', {'map': True}),
            'generate_diffs'             : 'tokenlist',
            'browser_base_url'           : ('unicode',
                                           {'subst': True, 'map': True}),
            'long_mail_action'           : ('mailaction', {'map': True}),
            'long_news_action'           : ('mailaction', {'map': True}),
            'mail_type'                  : ('unicode',    {'map': True}),
            'mail_transfer_encoding'     : 'unicode',
            'news_transfer_encoding'     : 'unicode',
            'custom_header'              : ('unicode',
                                           {'subst': True, 'map': True}),
            'extract_x509_author'        : 'humanbool',
            'cia_rpc_server'             : ('unicode',    {'map': True}),
            'cia_project_name'           : ('unicode',
                                           {'subst': True, 'map': True}),
            'cia_project_module'         : ('unicode',
                                           {'subst': True, 'map': True}),
            'cia_project_branch'         : ('unicode',
                                           {'subst': True, 'map': True}),
            'cia_project_submodule'      : ('unicode',
                                           {'subst': True, 'map': True}),
            'cia_project_path'           : ('unicode',
                                           {'subst': True, 'map': True}),
            'apply_charset_property'     : 'humanbool',
            'show_applied_charset'       : ('token', {'allowed': (u'yes',
                                            u'no', u'nondefault')}),

            # deprecated
            'viewcvs_base_url'           : ('unicode',
                                           {'subst': True, 'map': True}),
        }
        result = self._member_args['the_members']
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03aliases(self):
        """ groupMembers passes the proper aliases """
        self._settings.groupMembers({})

        expected = {
            'suppress_if_match'  : 'ignore_if_other_matches',
            'fallback'           : 'ignore_if_other_matches',
            'reply_to'           : 'reply_to_addr',
            'x509_author'        : 'extract_x509_author',
            'charset_property'   : 'apply_charset_property',
            'truncate_subject'   : 'max_subject_length',
            'subject_length'     : 'max_subject_length',
            'diff'               : 'diff_command',
            'nonmatching_paths'  : 'show_nonmatching_paths',
            'nongroup_paths'     : 'show_nonmatching_paths',
            'show_nongroup_paths': 'show_nonmatching_paths',
        }
        result = self._member_args['aliases']
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test04return(self):
        """ groupMembers returns the typedstruct.members result """
        expected = ['a']
        result = self._settings.groupMembers({})
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(groupMembersTestCase)


class generalMembersTestCase(memberTestBase):
    """ svnmailer.settings.generalMembers """

    def test00space(self):
        """ generalMembers passes the space around """
        space = {'mocked': 'space'}
        self._settings.generalMembers(space)

        expected = {'mocked': 'space'}
        result = self._member_args['space']
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test01typemap(self):
        """ generalMembers passes struct_accessors.typemap """
        self._settings.generalMembers({})

        expected = {'mocked': 'typemap'}
        result = self._member_args['typemap']
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02members(self):
        """ generalMembers passes the proper member definitions """
        self._settings.generalMembers({})

        expected = {
            'sendmail_command'  : ('unicommand', {'map': True}),
            'smtp_host'         : ('unicode',    {'map': True}),
            'smtp_user'         : ('quotedstr',  {'map': True}),
            'smtp_pass'         : ('quotedstr',  {'map': True}),
            'nntp_host'         : ('unicode',    {'map': True}),
            'nntp_user'         : ('quotedstr',  {'map': True}),
            'nntp_pass'         : ('quotedstr',  {'map': True}),
            'debug_all_mails_to': ('tokenlist',  {'map': True}),
            'tempdir'           : ('filename',   {'map': True}),

            # deprecated
            'cia_rpc_server'    : ('unicode',    {'map': True}),
            'diff_command'      : ('unicommand', {'map': True}),
        }
        result = self._member_args['the_members']
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03aliases(self):
        """ generalMembers passes the proper aliases """
        self._settings.generalMembers({})

        expected = {
            'mail_command' : 'sendmail_command',
            'smtp_hostname': 'smtp_host',
            'diff'         : 'diff_command',
        }
        result = self._member_args['aliases']
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test04return(self):
        """ generalMembers returns the typedstruct.members result """
        expected = ['a']
        result = self._settings.generalMembers({})
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(generalMembersTestCase)


class runtimeMembersTestCase(memberTestBase):
    """ svnmailer.settings.runtimeMembers """

    def test00space(self):
        """ runtimeMembers passes the space around """
        space = {'mocked': 'space'}
        self._settings.runtimeMembers(space)

        expected = {'mocked': 'space'}
        result = self._member_args['space']
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test01typemap(self):
        """ runtimeMembers passes struct_accessors.typemap """
        self._settings.runtimeMembers({})

        expected = {'mocked': 'typemap'}
        result = self._member_args['typemap']
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02members(self):
        """ runtimeMembers passes the proper member definitions """
        self._settings.runtimeMembers({})

        expected = {
            '_repos'       : None,
            'stdin'        : 'stdin',
            'path_encoding': 'string',
            'debug'        : 'bool',
            'revision'     : 'int',
            'repository'   : 'filename',
            'config'       : 'filename',
            'mode'         : 'string',
            'author'       : 'unicode',
            'propname'     : 'unicode',
            'action'       : 'unicode',
        }
        result = self._member_args['the_members']
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03aliases(self):
        """ runtimeMembers passes the proper aliases """
        self._settings.runtimeMembers({})

        expected = None
        result = self._member_args['aliases']
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test04return(self):
        """ runtimeMembers returns the typedstruct.members result """
        expected = ['a']
        result = self._settings.runtimeMembers({})
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(runtimeMembersTestCase)


class GroupSettingsContainerTestCase(containerTestBase):
    """ svnmailer.settings.GroupSettingsContainer """
    _members_return = ['groupsettings']

    def test00super(self):
        """ GroupSettingsContainer inherits from typedstruct.Struct """
        expected = "magic"
        result = self._settings.GroupSettingsContainer._mocked_object
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test01slots(self):
        """ GroupSettingsContainer.__slots__ is set from groupMembers """
        expected = ['groupsettings']
        result = self._settings.GroupSettingsContainer.__slots__
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

        space = {}
        self._settings.groupMembers(space)
        expected = space['_member_args']
        result = self._settings.GroupSettingsContainer._member_args
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02getIgnoreableMembers(self):
        """ GroupSettingsContainer._getIgnorableMembers returns proper data """
        inst = self._settings.GroupSettingsContainer()

        expected = [
            '_name', '_def_for_repos', '_def_for_paths',
            'for_repos', 'for_paths', 'exclude_paths',
            'ignore_if_other_matches', 'to_addr', 'from_addr',
            'to_newsgroup', 'custom_header', 'cia_rpc_server',
            'cia_project_name', 'cia_project_module', 'cia_project_branch',
            'cia_project_submodule', 'cia_project_path',
        ]
        result = inst._getIgnorableMembers()
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03compare(self):
        """ GroupSettingsContainer._compare rejects foreign types """
        inst = self._settings.GroupSettingsContainer()

        expected = False
        result = inst._compare(self)
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test04compare(self):
        """ GroupSettingsContainer._compare compares the proper attributes """
        class MyContainer(self._settings.GroupSettingsContainer, object):
            _members_ = ('a', 'b', 'c')
            def _getIgnorableMembers(self):
                return ['a', 'c']
            def set(self, a, b, c):
                self.a = a
                self.b = b
                self.c = c

        inst1 = MyContainer()
        inst2 = MyContainer()

        tests = (
            (True,  (1, 1, 1), (1, 1, 1)),
            (True,  (1, 1, 1), (2, 1, 1)),
            (True,  (1, 1, 1), (2, 1, 2)),
            (True,  (1, 1, 1), (2, 1, 3)),
            (False, (1, 2, 1), (1, 1, 1)),
            (False, (1, 2, 1), (2, 1, 1)),
            (False, (1, 2, 1), (2, 1, 2)),
            (False, (1, 2, 1), (2, 1, 3)),
            (False, (1, 4, 1), (2, 1, 3)),
        )
        for expected, set1, set2 in tests:
            inst1.set(*set1)
            inst2.set(*set2)
            result = inst1._compare(inst2)
            self.failUnlessEqual(expected, result,
                "Input %r expected %r, got %r" %
                ((set1, set2), expected, result)
            )
            result = inst2._compare(inst1)
            self.failUnlessEqual(expected, result,
                "Input %r expected %r, got %r" %
                ((set1, set2), expected, result)
            )

_tests.append(GroupSettingsContainerTestCase)


class GeneralSettingsContainerTestCase(containerTestBase):
    """ svnmailer.settings.GeneralSettingsContainer """
    _members_return = ['generalsettings']

    def test00super(self):
        """ GeneralSettingsContainer inherits from typedstruct.Struct """
        expected = "magic"
        result = self._settings.GeneralSettingsContainer._mocked_object
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test01slots(self):
        """ GeneralSettingsContainer.__slots__ is set from generalMembers """
        expected = ['generalsettings']
        result = self._settings.GeneralSettingsContainer.__slots__
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

        space = {}
        self._settings.generalMembers(space)
        expected = space['_member_args']
        result = self._settings.GeneralSettingsContainer._member_args
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(GeneralSettingsContainerTestCase)


class RuntimeSettingsContainerTestCase(containerTestBase):
    """ svnmailer.settings.RuntimeSettingsContainer """
    _members_return = ['runtimesettings']

    def test00super(self):
        """ RuntimeSettingsContainer inherits from typedstruct.Struct """
        expected = "magic"
        result = self._settings.RuntimeSettingsContainer._mocked_object
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test01slots(self):
        """ RuntimeSettingsContainer.__slots__ is set from runtimeMembers """
        expected = ['runtimesettings']
        result = self._settings.RuntimeSettingsContainer.__slots__
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

        space = {}
        self._settings.runtimeMembers(space)
        expected = space['_member_args']
        result = self._settings.RuntimeSettingsContainer._member_args
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(RuntimeSettingsContainerTestCase)


class SettingsTestCase(settingsTestBase):
    """ svnmailer.settings.Settings """

    def test00init(self):
        """ Settings is initialized properly """
        import sys
        expected = ["init", "us-ascii", None, None, [], None, None, "check"]
        result = []
        class MySettings(self._settings.Settings):
            def _checkInitialization(this):
                result.append('check')
            def init(this, *args, **kwargs):
                result.append('init')
                result.extend([
                    this._charset_, this._fcharset_, this._maps_,
                    this.groups, this.general, this.runtime
                ])

        inst = MySettings()
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test01init(self):
        """ Settings passes all call arguments to the custom initializer """
        expected = [('1', '2'), {'foo': self}]
        result = []
        class MySettings(self._settings.Settings):
            def _checkInitialization(this):
                pass
            def init(self, *args, **kwargs):
                result.extend([args, kwargs])

        inst = MySettings('1', '2', foo = self)
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02init(self):
        """ Settings.init is not implemented """
        try:
            self._settings.Settings()
        except NotImplementedError:
            # cool
            pass
        else:
            self.fail("Doesn't raise NotImplementedError")

    def test03check(self):
        """ Settings._checkInitialization raises RuntimeError correctly """
        class MySettings(self._settings.Settings):
            def init(this, general, runtime, groups):
                this.general = general
                this.runtime = runtime
                this.groups = groups

        tests = (
            (True,  (None, None, None)),
            (True,  (1,    None, None)),
            (True,  (None, 1,    None)),
            (True,  (None, None, 1   )),
            (True,  (1,    1,    None)),
            (True,  (1,    None, 1   )),
            (True,  (None, 1,    1   )),
            (False, (1,    1,    1   )),
        )
        for expected, test in tests:
            try:
                MySettings(*test)
            except RuntimeError:
                if not expected:
                    self.fail("Input %r raises RuntimeError" % (test,))
            else:
                if expected:
                    self.fail("Input %r doesn't raise RuntimeError" % (test,))

    def test04getGroupContainer(self):
        """ Settings.getGroupContainer returns an initialized container """
        class MySettings(self._settings.Settings):
            def _checkInitialization(this):
                pass
            def init(this, *args, **kwargs):
                pass

        # pylint-0.6.4-anti-crash hack (indirection via self)
        self.__base_container = self._settings.GroupSettingsContainer
        class MyContainer(self.__base_container):
            def __init__(this, _maps_, _arg_, **kwargs):
                this.args = [_maps_, _arg_, kwargs]
        self._settings.GroupSettingsContainer = MyContainer

        inst = MySettings()
        inst._maps_ = 'foo'
        inst._charset_ = 'bar'
        inst._fcharset_ = 'fbar'
        cont = inst.getGroupContainer(baz = "zonk")
        self.failUnless(isinstance(cont, self.__base_container),
            "Not the proper container, but %r" % (cont,)
        )

        expected = [
            'foo', {'encoding': 'bar', 'path_encoding': 'fbar'},
            {'baz': 'zonk'}
        ]
        result = cont.args
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test05getDefaultGroupContainer(self):
        " Settings.getDefaultGroupContainer returns an initialized container "
        class MySettings(self._settings.Settings):
            def _checkInitialization(this):
                pass
            def init(this, *args, **kwargs):
                pass

        # pylint-0.6.4-anti-crash hack (indirection via self)
        self.__base_container = self._settings.GroupSettingsContainer
        class MyContainer(self.__base_container):
            def __init__(this, _maps_, _arg_, **kwargs):
                this.args = [_maps_, _arg_, kwargs]
        self._settings.GroupSettingsContainer = MyContainer

        inst = MySettings()
        inst._maps_ = 'foo'
        inst._charset_ = 'bar'
        inst._fcharset_ = 'fbar'
        cont = inst.getDefaultGroupContainer(baz = "zonk")
        self.failUnless(isinstance(cont, self.__base_container),
            "Not the proper container, but %r" % (cont,)
        )

        expected = [
            None, {'encoding': 'bar', 'path_encoding': 'fbar'},
            {'baz': 'zonk'}
        ]
        result = cont.args
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test06getGeneralContainer(self):
        """ Settings.getGeneralContainer returns an initialized container """
        class MySettings(self._settings.Settings):
            def _checkInitialization(this):
                pass
            def init(this, *args, **kwargs):
                pass

        # pylint hack
        self.__base_container = self._settings.GeneralSettingsContainer
        class MyContainer(self.__base_container):
            def __init__(this, _maps_, _arg_, **kwargs):
                this.args = [_maps_, _arg_, kwargs]
        self._settings.GeneralSettingsContainer = MyContainer

        inst = MySettings()
        inst._maps_ = 'foo2'
        inst._charset_ = 'bar2'
        inst._fcharset_ = 'fbar2'
        cont = inst.getGeneralContainer(baz2 = "zonk2")
        self.failUnless(isinstance(cont, self.__base_container),
            "Not the proper container, but %r" % (cont,)
        )

        expected = [
            'foo2', {'encoding': 'bar2', 'path_encoding': 'fbar2'},
            {'baz2': 'zonk2'}
        ]
        result = cont.args
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test07getRuntimeContainer(self):
        """ Settings.getRuntimeContainer returns an initialized container """
        class MySettings(self._settings.Settings):
            def _checkInitialization(this):
                pass
            def init(this, *args, **kwargs):
                pass

        # pylint hack
        self.__base_container = self._settings.RuntimeSettingsContainer
        class MyContainer(self.__base_container):
            def __init__(this, _maps_, _arg_, **kwargs):
                this.args = [_maps_, _arg_, kwargs]
        self._settings.RuntimeSettingsContainer = MyContainer

        inst = MySettings()
        inst._maps_ = 'foo3'
        inst._charset_ = 'bar3'
        inst._fcharset_ = 'fbar3'
        cont = inst.getRuntimeContainer(baz3 = "zonk3")
        self.failUnless(isinstance(cont, self.__base_container),
            "Not the proper container, but %r" % (cont,)
        )

        expected = [
            None, {'encoding': 'utf-8', 'path_encoding': 'fbar3'},
            {'baz3': 'zonk3'}
        ]
        result = cont.args
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(SettingsTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return unittest.TestSuite([
        unittest.makeSuite(test) for test in _tests
    ])

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity = 2).run(getSuite())
