# -*- coding: utf-8 -*-
# pylint: disable-msg = C0103, W0232, W0302, W0612, W0613, W0703, W0704, E0213
#
# Copyright 2004-2006 André Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.struct_accessors

@var _tests: The test classes
@type _tests: C{list}
"""
__author__    = "André Malo"
__docformat__ = "epytext en"
__all__       = ['getSuite']

import unittest
_tests = []


class importTestCase(unittest.TestCase):
    """ Importing svnmailer.struct_accessors """

    def testImport(self):
        """ from svnmailer import struct_accessors """
        try:
            from svnmailer import struct_accessors
        except Exception, exc:
            self.fail(
                "svnmailer.struct_accessors could not be imported: %s" %
                str(exc)
            )

_tests.append(importTestCase)


class accessorTestBase(unittest.TestCase):
    """ Base class for descriptor tests """

    def setUp(self):
        """ Creates test environment """
        from svnmailer import struct_accessors, typedstruct

        self._acc = struct_accessors
        self._priv = typedstruct.StructPrivate(['a'], {})


class BaseDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.BaseDescriptor """

    def test00init(self):
        """ BaseDescriptor.__init__ initializes .param as dict """
        inst = self._acc.BaseDescriptor('a', self._priv)

        expected = {}
        result = inst.param
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test01init(self):
        """ BaseDescriptor.__init__ initializes .MAP and .SUBST from param """
        tests = (
            (None,                             (False, False)),
            ({'map': 0},                       (False, False)),
            ({'map': 1},                       (True,  False)),
            ({'map': 1, 'subst': 0},           (True,  False)),
            ({'map': 1, 'subst': 1},           (True,  True )),
            ({'map': 1, 'subst': 1, 'a': 'b'}, (True,  True )),
            ({'map': 0, 'subst': 1, 'a': 'b'}, (False, True )),
            ({'subst': 1},                     (False, True )),
            ({'subst': 0},                     (False, False)),
            ({},                               (False, False)),
        )

        for test, expected in tests:
            inst = self._acc.BaseDescriptor('a', self._priv, test)
            result = (inst.MAP, inst.SUBST)
            self.failUnlessEqual(expected, result,
                "Input %r expected %r, got %r" % (test, expected, result)
            )

    def test02transform(self):
        """ BaseDescriptor.transform passes through None values """
        class MyDescriptor(self._acc.BaseDescriptor):
            def doTransform(this, value, arg):
                self.fail("doTransform was called")
        inst = MyDescriptor('a', self._priv) 

        expected = None
        result = inst.transform(None, "foo")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03transform(self):
        """ BaseDescriptor.transform calls doTransform """
        class MyDescriptor(self._acc.BaseDescriptor):
            def doTransform(this, value, arg):
                return (value, arg)
        inst = MyDescriptor('a', self._priv)

        expected = ("foo", "bar")
        result = inst.transform("foo", "bar")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test04substitute(self):
        """ BaseDescriptor.substitute passes through None values """
        class MyDescriptor(self._acc.BaseDescriptor):
            def doSubstitute(this, value, subst, arg):
                self.fail("doSubstitute was called")
        inst = MyDescriptor('a', self._priv)

        expected = None
        result = inst.substitute(None, {'a': 'b'}, "foo")
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        inst = MyDescriptor('a', self._priv, {'subst': True})
        expected = None
        result = inst.substitute(None, {'a': 'b'}, "foo")
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test05substitute(self):
        """ BaseDescriptor.substitute passes through if not activated """
        class MyDescriptor(self._acc.BaseDescriptor):
            def doSubstitute(this, value, subst, arg):
                self.fail("doSubstitute was called")
        inst = MyDescriptor('a', self._priv)

        expected = "bar"
        result = inst.substitute("bar", {'a': 'b'}, "foo")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test06substitute(self):
        """ BaseDescriptor.substitute calls doSubstitute """
        class MyDescriptor(self._acc.BaseDescriptor):
            def doSubstitute(this, value, subst, arg):
                return (value, subst, arg)
        inst = MyDescriptor('a', self._priv, {'subst': True})

        expected = ("foo", {'a': 'b'}, "bar")
        result = inst.substitute("foo", {'a': 'b'}, "bar")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test07premap(self):
        """ BaseDescriptor.premap passes through None values """
        class MyDescriptor(self._acc.BaseDescriptor):
            def doPremap(this, value, mapper, arg):
                self.fail("doPremap was called")
        inst = MyDescriptor('a', self._priv)

        expected = None
        result = inst.premap(None, "foo", "bar")
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        inst = MyDescriptor('a', self._priv, {'map': True})
        expected = None
        result = inst.premap(None, "foo", "bar")
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test08premap(self):
        """ BaseDescriptor.premap passes through if not activated """
        class MyDescriptor(self._acc.BaseDescriptor):
            def doPremap(this, value, mapper, arg):
                self.fail("doPremap was called")
        inst = MyDescriptor('a', self._priv)

        expected = "baz"
        result = inst.premap("baz", "foo", "bar")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test09premap(self):
        """ BaseDescriptor.premap calls doPremap """
        class MyDescriptor(self._acc.BaseDescriptor):
            def doPremap(this, value, mapper, arg):
                return (value, mapper, arg)
        inst = MyDescriptor('a', self._priv, {'map': True})

        expected = ("foo", "bar", "baz")
        result = inst.premap("foo", "bar", "baz")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test10postmap(self):
        """ BaseDescriptor.postmap passes through None values """
        class MyDescriptor(self._acc.BaseDescriptor):
            def doPostmap(this, value, mapper, arg):
                self.fail("doPostmap was called")
        inst = MyDescriptor('a', self._priv)

        expected = None
        result = inst.postmap(None, "foo", "bar")
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        inst = MyDescriptor('a', self._priv, {'map': True})
        expected = None
        result = inst.postmap(None, "foo", "bar")
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test11postmap(self):
        """ BaseDescriptor.postmap passes through if not activated """
        class MyDescriptor(self._acc.BaseDescriptor):
            def doPostmap(this, value, mapper, arg):
                self.fail("doPostmap was called")
        inst = MyDescriptor('a', self._priv)

        expected = "baz"
        result = inst.postmap("baz", "foo", "bar")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test12postmap(self):
        """ BaseDescriptor.postmap calls doPostmap """
        class MyDescriptor(self._acc.BaseDescriptor):
            def doPostmap(this, value, mapper, arg):
                return (value, mapper, arg)
        inst = MyDescriptor('a', self._priv, {'map': True})

        expected = ("foo", "bar", "baz")
        result = inst.postmap("foo", "bar", "baz")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test13getCharset(self):
        """ BaseDescriptor.getCharset returns arg or charset """
        inst = self._acc.BaseDescriptor('a', self._priv)

        expected = "utf-8"
        result = inst.getCharset({'encoding': "utf-8"})
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        expected = "us-ascii"
        result = inst.getCharset(None)
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test14getFileCharset(self):
        """ BaseDescriptor.getFileCharset returns arg or charset """
        inst = self._acc.BaseDescriptor('a', self._priv)

        expected = "utf-8"
        result = inst.getFileCharset({'path_encoding': "utf-8"})
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        expected = None
        result = inst.getFileCharset(None)
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test15doPremap(self):
        """ BaseDescriptor.doPremap is not implemented """
        inst = self._acc.BaseDescriptor('a', self._priv)

        try:
            inst.doPremap("foo", "bar", "baz")
        except NotImplementedError:
            # cool
            pass
        else:
            self.fail("Doesn't raise NotImplementedError")

    def test16doTransform(self):
        """ BaseDescriptor.doTransform is not implemented """
        inst = self._acc.BaseDescriptor('a', self._priv)

        try:
            inst.doTransform("foo", "bar")
        except NotImplementedError:
            # cool
            pass
        else:
            self.fail("Doesn't raise NotImplementedError")

    def test17doSubstitute(self):
        """ BaseDescriptor.doSubstitute is not implemented """
        inst = self._acc.BaseDescriptor('a', self._priv)

        try:
            inst.doSubstitute("foo", "bar", "baz")
        except NotImplementedError:
            # cool
            pass
        else:
            self.fail("Doesn't raise NotImplementedError")

    def test18doPostmap(self):
        """ BaseDescriptor.doPostmap is not implemented """
        inst = self._acc.BaseDescriptor('a', self._priv)

        try:
            inst.doPostmap("foo", "bar", "baz")
        except NotImplementedError:
            # cool
            pass
        else:
            self.fail("Doesn't raise NotImplementedError")

    def test19Descriptor(self):
        """ BaseDescriptor has __set__, __get__ and __delete__ """
        inst = self._acc.BaseDescriptor('a', self._priv)

        expected = dict.fromkeys(['s', 'g', 'd'], True)
        result = {
            's': hasattr(inst, '__set__'),
            'g': hasattr(inst, '__get__'),
            'd': hasattr(inst, '__delete__'),
        }
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(BaseDescriptorTestCase)


class BasePremapDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.BasePremapDescriptor """

    def test00Descriptor(self):
        """ BasePremapDescriptor is a BaseDescriptor """
        inst = self._acc.BasePremapDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BaseDescriptor),
            "Is not a BaseDescriptor"
        )

    def test01doPremap(self):
        """ BasePremapDescriptor implements a default premapper """
        inst = self._acc.BasePremapDescriptor('a', self._priv)
        def mapper(value):
            return ("mapper", value)

        expected = ("mapper", "foo")
        result = inst.doPremap("foo", mapper, "bar")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02doPostmap(self):
        """ BasePremapDescriptor implements a pass-through postmapper """
        inst = self._acc.BasePremapDescriptor('a', self._priv)
        def mapper(value):
            return ("mapper", value)

        expected = "foo"
        result = inst.doPostmap("foo", mapper, "bar")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(BasePremapDescriptorTestCase)


class BasePostmapDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.BasePostmapDescriptor """

    def test00Descriptor(self):
        """ BasePostmapDescriptor is a BaseDescriptor """
        inst = self._acc.BasePostmapDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BaseDescriptor),
            "Is not a BaseDescriptor"
        )

    def test01doPremap(self):
        """ BasePostmapDescriptor implements a pass-through premapper """
        inst = self._acc.BasePostmapDescriptor('a', self._priv)
        def mapper(value):
            return ("mapper", value)

        expected = "foo"
        result = inst.doPremap("foo", mapper, "bar")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(BasePostmapDescriptorTestCase)


class UnicodeDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.UnicodeDescriptor """

    def test00Descriptor(self):
        """ UnicodeDescriptor is a BasePostmapDescriptor """
        inst = self._acc.UnicodeDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BasePostmapDescriptor),
            "Is not a BasePostmapDescriptor"
        )

    def test01doTransform(self):
        """ UnicodeDescriptor.doTransform rejects non string/unicode values """
        inst = self._acc.UnicodeDescriptor('a', self._priv)
        try:
            inst.doTransform(self, "arg")
        except TypeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise TypeError")

    def test02doTransform(self):
        """ UnicodeDescriptor.doTransform passes through unicode """
        inst = self._acc.UnicodeDescriptor('a', self._priv)

        expected = u"Andr\xe9"
        result = inst.doTransform(u"Andr\xe9", None)
        self.failUnless(
            type(expected) == type(result) and expected == result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03doTransform(self):
        """ UnicodeDescriptor.doTransform decodes string values """
        inst = self._acc.UnicodeDescriptor('a', self._priv)

        expected = u"Andr\xe9"
        result = inst.doTransform("Andr\xe9", {'encoding': "iso-8859-1"})
        self.failUnless(
            type(expected) == type(result) and expected == result,
            "Expected %r, got %r" % (expected, result)
        )

    def test04doTransform(self):
        """ UnicodeDescriptor.doTransform raises a UnicodeError """
        inst = self._acc.UnicodeDescriptor('a', self._priv)

        try:
            inst.doTransform("Andr\xe9", None)
        except UnicodeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise UnicodeError")

    def test05doSubstitute(self):
        """ UnicodeDescriptor.doSubstitute subsitutes stuff in values """
        inst = self._acc.UnicodeDescriptor('a', self._priv)

        expected = u"foo"
        result = inst.doSubstitute(u"f%(key1)s%(key2)s", {'key1': 'oo'}, "bar")
        self.failUnless(
            type(expected) == type(result) and expected == result,
            "Expected %r, got %r" % (expected, result)
        )

    def test06doPostmap(self):
        """ UnicodeDescriptor.doPostmap decodes string mapper results """
        inst = self._acc.UnicodeDescriptor('a', self._priv)

        def mapper(value):
            return value

        expected = u"foo"
        result = inst.doPostmap("foo", mapper, None)
        self.failUnless(
            type(expected) == type(result) and expected == result,
            "Expected %r, got %r" % (expected, result)
        )

    def test07doPostmap(self):
        """ UnicodeDescriptor.doPostmap raises UnicodeError """
        inst = self._acc.UnicodeDescriptor('a', self._priv)

        def mapper(value):
            return value

        try:
            inst.doPostmap("Andr\xe9", mapper, None)
        except UnicodeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise UnicodeError")

    def test08doPostmap(self):
        """ UnicodeDescriptor.doPostmap passes None values """
        inst = self._acc.UnicodeDescriptor('a', self._priv)

        def mapper(value):
            return value

        expected = None
        result = inst.doPostmap(None, mapper, "foo")
        self.failUnless(
            type(expected) == type(result) and expected == result,
            "Expected %r, got %r" % (expected, result)
        )

    def test09doPostmap(self):
        """ UnicodeDescriptor.doPostmap passes Unicode """
        inst = self._acc.UnicodeDescriptor('a', self._priv)

        def mapper(value):
            return value

        expected = u"Andr\xe9"
        result = inst.doPostmap(u"Andr\xe9", mapper, None)
        self.failUnless(
            type(expected) == type(result) and expected == result,
            "Expected %r, got %r" % (expected, result)
        )

    def test10doPostmap(self):
        """ UnicodeDescriptor.doPostmap calls the mapper function """
        inst = self._acc.UnicodeDescriptor('a', self._priv)

        def mapper(value):
            return "Andr\xe9"

        expected = u"Andr\xe9"
        result = inst.doPostmap("foo", mapper, {'encoding': 'iso-8859-1'})
        self.failUnless(
            type(expected) == type(result) and expected == result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(UnicodeDescriptorTestCase)


class StringDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.StringDescriptor """

    def test00Descriptor(self):
        """ StringDescriptor is a BaseDescriptor """
        inst = self._acc.StringDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.StringDescriptor),
            "Is not a BaseDescriptor"
        )

    def test01doTransform(self):
        """ StringDescriptor.doTransform accepts random types """
        inst = self._acc.StringDescriptor('a', self._priv)
        inst.doTransform(self, "foo")
        inst.doTransform("bar", "foo")

    def test02doTransform(self):
        """ StringDescriptor converts to string """
        tests = (self, "foo", 1)
        inst = self._acc.StringDescriptor('a', self._priv)

        expected = str
        for test in tests:
            result = inst.doTransform(test, None)
            self.failUnless(isinstance(result, expected),
                "Expected %r, got %r" % (expected, result)
            )

_tests.append(StringDescriptorTestCase)


class IntegerDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.IntegerDescriptor """

    def test00Descriptor(self):
        """ IntegerDescriptor is a BaseDescriptor """
        inst = self._acc.IntegerDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BaseDescriptor),
            "Is not a BaseDescriptor"
        )

    def test01doTransform(self):
        """ IntegerDescriptor.doTransform rejects unconvertable values """
        inst = self._acc.IntegerDescriptor('a', self._priv)
        try:
            inst.doTransform(self, "foo")
        except TypeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise TypeError")

        try:
            inst.doTransform("bar", "foo")
        except ValueError:
            # cool
            pass
        else:
            self.fail("Doesn't raise ValueError")

    def test02doTransform(self):
        """ IntegerDescriptor converts to integers """
        inst = self._acc.IntegerDescriptor('a', self._priv)

        expected = 10
        result = inst.doTransform("10", None)
        self.failUnless(
            type(expected) == type(result) and expected == result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(IntegerDescriptorTestCase)


class BooleanDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.BooleanDescriptor """

    def test00Descriptor(self):
        """ BooleanDescriptor is a BaseDescriptor """
        inst = self._acc.BooleanDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BaseDescriptor),
            "Is not a BaseDescriptor"
        )

    def test01doTransform(self):
        """ BooleanDescriptor.doTransform accepts random types """
        inst = self._acc.BooleanDescriptor('a', self._priv)
        inst.doTransform(self, "foo")
        inst.doTransform("bar", "foo")

    def test02doTransform(self):
        """ BooleanDescriptor converts to boolean """
        tests = (
            (True,  "10" ),
            (True,  "0"  ),
            (False, ""   ),
            (False, False),
        )
        inst = self._acc.BooleanDescriptor('a', self._priv)

        for expected, test in tests:
            result = inst.doTransform(test, None)
            self.failUnless(
                type(expected) == type(result) and expected == result,
                "Expected %r, got %r" % (expected, result)
            )

_tests.append(BooleanDescriptorTestCase)


class HumanBooleanDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.HumanBooleanDescriptor """

    def test00Descriptor(self):
        """ HumanBooleanDescriptor is a BaseDescriptor """
        inst = self._acc.HumanBooleanDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BaseDescriptor),
            "Is not a BaseDescriptor"
        )

    def test01Mutex(self):
        """ HumanBooleanDescriptor defines mutual exclusive keywords """
        cls = self._acc.HumanBooleanDescriptor
        self.failUnless(cls.TRUE, "Defines no TRUE keywords")
        self.failUnless(cls.FALSE, "Defines no FALSE keywords")

        for word in cls.TRUE:
            self.failIf(word in cls.FALSE,
                "%r is defined in TRUE and FALSE" % word
            )

    def test02doTransform(self):
        """ HumanBooleanDescriptor transforms to the proper boolean """
        inst = self._acc.HumanBooleanDescriptor('a', self._priv)

        for expected, words in ((True, inst.TRUE), (False, inst.FALSE)):
            for word in words:
                for real in (word, word.upper(), word.lower()):
                    result = inst.doTransform(real, "foo")
                    self.failUnlessEqual(expected, result,
                        "Input %r expected %r, got %r" %
                        (real, expected, result)
                    )

    def test03doTransform(self):
        """ HumanBooleanDescriptor rejects unknown values """
        inst = self._acc.HumanBooleanDescriptor('a', self._priv)

        try:
            inst.doTransform(str(self), "foo")
        except ValueError:
            # cool
            pass
        else:
            self.fail("Doesn't raise ValueError")

_tests.append(HumanBooleanDescriptorTestCase)


class RegexDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.RegexDescriptor """

    def test00Descriptor(self):
        """ RegexDescriptor is a BasePremapDescriptor """
        inst = self._acc.RegexDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BasePremapDescriptor),
            "Is not a BasePremapDescriptor"
        )

    def test01init(self):
        """ RegexDescriptor.__init__ sets FLAGS from .param """
        inst = self._acc.RegexDescriptor('a', self._priv, {'flags': 10})

        expected = 10
        result = inst.FLAGS
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02doTransform(self):
        """ RegexDescriptor.doTransform rejects non string/unicode types """
        inst = self._acc.RegexDescriptor('a', self._priv)

        try:
            inst.doTransform(self, None)
        except TypeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise TypeError")

    def test03doTransform(self):
        """ RegexDescriptor.doTransform passes regex objects """
        import re
        inst = self._acc.RegexDescriptor('a', self._priv)

        expected = re.compile(r'abc')
        result = inst.doTransform(expected, "foo")
        self.failUnless(expected is result,
            "Expected %r, got %r" % (expected, result)
        )

    def test04doTransform(self):
        """ RegexDescriptor.doTransform decodes strings """
        inst = self._acc.RegexDescriptor('a', self._priv)

        expected = u"Andr\xe9"
        result = inst.doTransform(
            "Andr\xe9", {'encoding': "iso-8859-1"}
        ).pattern
        self.failUnless(
            type(expected) == type(result) and expected == result,
            "Expected %r, got %r" % (expected, result)
        )

    def test05doTransform(self):
        """ RegexDescriptor.doTransform raises UnicodeError """
        inst = self._acc.RegexDescriptor('a', self._priv)

        try:
            inst.doTransform("Andr\xe9", None)
        except UnicodeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise UnicodeError")

    def test06doTransform(self):
        """ RegexDescriptor.doTransform compiles a usable regex """
        inst = self._acc.RegexDescriptor('a', self._priv)

        expected = u"b"
        result = inst.doTransform(r'a(b)c', None).match(u'abc').group(1)
        self.failUnless(
            type(expected) == type(result) and expected == result,
            "Expected %r, got %r" % (expected, result)
        )

    def test07doTransform(self):
        """ RegexDescriptor.doTransform rejects invalid regexps """
        inst = self._acc.RegexDescriptor('a', self._priv)

        try:
            inst.doTransform("*", None)
        except ValueError:
            # cool
            pass
        else:
            self.fail("Doesn't raise ValueError")

_tests.append(RegexDescriptorTestCase)


class TokenDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.TokenDescriptor """

    def test00Descriptor(self):
        """ TokenDescriptor is a BasePremapDescriptor """
        inst = self._acc.TokenDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BasePremapDescriptor),
            "Is not a BasePremapDescriptor"
        )

    def test01init(self):
        """ TokenDescriptor.__init__ initializes .ALLOWED from param """
        inst = self._acc.TokenDescriptor('a', self._priv)
        expected = None
        result = inst.ALLOWED
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        inst = self._acc.TokenDescriptor(
            'a', self._priv, {'allowed': ['b']}
        )
        expected = {'b': None}
        result = inst.ALLOWED
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test02doTransform(self):
        """ TokenDescriptor.doTransform decodes strings input """
        inst = self._acc.TokenDescriptor('a', self._priv)

        expected = (u"andr\xe9 malo")
        result = inst.doTransform(
            "Andr\xe9 Malo", {'encoding': "iso-8859-1"}
        )
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03doTransform(self):
        """ TokenDescriptor.doTransform raises UnicodeError """
        inst = self._acc.TokenDescriptor('a', self._priv)

        try:
            inst.doTransform("Andr\xe9", None)
        except UnicodeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise UnicodeError")

    def test04doTransform(self):
        """ TokenDescriptor.doTransform raises on non-string/unicode types """
        inst = self._acc.TokenDescriptor('a', self._priv)

        try:
            inst.doTransform(self, None)
        except TypeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise TypeError")

    def test05doTransform(self):
        """ TokenDescriptor.doTransform checks string/unicode input """
        tests = ["1 a", u"a", " a 1  ", u"a   b", "  a b     1", u"a \tb   c"]
        inst = self._acc.TokenDescriptor('a', self._priv, {'allowed': '1'})

        for test in tests:
            try:
                inst.doTransform(test, {'encoding': "iso-8859-1"})
            except ValueError:
                # cool
                pass
            else:
                self.fail("Input %r doesn't raise ValueError" % (test,))

    def test06doTransform(self):
        """ TokenDescriptor.doTransform passes allowed string/unicode input """
        tests = ["1", u"1"]
        inst = self._acc.TokenDescriptor('a', self._priv, {'allowed': u'1'})

        for test in tests:
            inst.doTransform(test, {'encoding': "iso-8859-1"})

_tests.append(TokenDescriptorTestCase)


class TokenlistDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.TokenlistDescriptor """

    def test00Descriptor(self):
        """ TokenlistDescriptor is a BasePostmapDescriptor """
        inst = self._acc.TokenlistDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BasePostmapDescriptor),
            "Is not a BasePostmapDescriptor"
        )

    def test01init(self):
        """ TokenlistDescriptor.__init__ initializes .ALLOWED from param """
        inst = self._acc.TokenlistDescriptor('a', self._priv)
        expected = None
        result = inst.ALLOWED
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        inst = self._acc.TokenlistDescriptor(
            'a', self._priv, {'allowed': ['b']}
        )
        expected = {'b': None}
        result = inst.ALLOWED
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test02doTransform(self):
        """ TokenlistDescriptor.doTransform passes tuples """
        inst = self._acc.TokenlistDescriptor('a', self._priv)

        expected = ('a', 'b')
        result = inst.doTransform(('a', 'b'), "foo")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03doTransform(self):
        """ TokenlistDescriptor.doTransform checks tuples """
        tests = [
            ('1', 'a'), ('a',), ('a', '1'), ('a', 'b'), ('a', 'b', '1'),
            ('a', 'b', 'c')
        ]
        inst = self._acc.TokenlistDescriptor('a', self._priv, {'allowed': '1'})

        for test in tests:
            try:
                inst.doTransform(test, "foo")
            except ValueError:
                # cool
                pass
            else:
                self.fail("Input %r doesn't raise ValueError" % (test,))

    def test04doTransform(self):
        """ TokenlistDescriptor.doTransform passes checked tuples """
        inst = self._acc.TokenlistDescriptor(
            'a', self._priv, {'allowed': ['a', 'b', 'c']}
        )

        expected = ('a', 'b')
        result = inst.doTransform(('a', 'b'), "foo")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test05doTransform(self):
        """ TokenlistDescriptor.doTransform splits unicode on whitespace """
        inst = self._acc.TokenlistDescriptor('a', self._priv)

        expected = (u'a', u'b')
        result = inst.doTransform(u" a \tb ", "foo")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test06doTransform(self):
        """ TokenlistDescriptor.doTransform decodes strings input """
        inst = self._acc.TokenlistDescriptor('a', self._priv)

        expected = (u"Andr\xe9", u"Malo")
        result = inst.doTransform(
            " Andr\xe9 Malo ", {'encoding': "iso-8859-1"}
        )
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test07doTransform(self):
        """ TokenlistDescriptor.doTransform raises UnicodeError """
        inst = self._acc.TokenlistDescriptor('a', self._priv)

        try:
            inst.doTransform("Andr\xe9", None)
        except UnicodeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise UnicodeError")

    def test08doTransform(self):
        " TokenlistDescriptor.doTransform raises on non-string/unicode types "
        inst = self._acc.TokenlistDescriptor('a', self._priv)

        try:
            inst.doTransform(self, None)
        except TypeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise TypeError")

    def test09doTransform(self):
        """ TokenlistDescriptor.doTransform checks string/unicode input """
        tests = ["1 a", u"a", " a 1  ", u"a   b", "  a b     1", u"a \tb   c"]
        inst = self._acc.TokenlistDescriptor('a', self._priv, {'allowed': '1'})

        for test in tests:
            try:
                inst.doTransform(test, {'encoding': "iso-8859-1"})
            except ValueError:
                # cool
                pass
            else:
                self.fail("Input %r doesn't raise ValueError" % (test,))

    def test10doSubstitute(self):
        """ TokenlistDescriptor.doSubstitute substitutes each token """
        inst = self._acc.TokenlistDescriptor('a', self._priv)

        expected = ('foo', 'bar', 'baz')
        result = inst.doSubstitute(
            (u'f%(key1)s%(key2)s', u'%(key3)s', u'%(key4)sbaz'),
            {'key1': u'oo', 'key3': u'bar'}, "foo"
        )
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test11doPostmap(self):
        """ TokenlistDescriptor.doPostmap maps each token """
        inst = self._acc.TokenlistDescriptor('a', self._priv)

        def mapper(token):
            return u'mapped-' + token

        expected = (u'mapped-foo', u'mapped-bar', u'mapped-baz')
        result = inst.doPostmap((u'foo', u'bar', u'baz'), mapper, "foo")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )
 
_tests.append(TokenlistDescriptorTestCase)


class FilenameDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.FilenameDescriptor """

    def test00Descriptor(self):
        """ FilenameDescriptor is a BasePremapDescriptor """
        inst = self._acc.FilenameDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BasePremapDescriptor),
            "Is not a BasePremapDescriptor"
        )

    def test01doTransform(self):
        """ FilenameDescriptor.doTransform recodes filenames """
        from svnmailer import util

        class MyDescriptor(self._acc.FilenameDescriptor):
            def getCharset(this, arg):
                return "iso-8859-15"
            def getFileCharset(this, arg):
                return "utf-8"

        inst = MyDescriptor('a', self._priv)

        expected = util.filename.unicode_system and \
            u"\u20acuro" or "\xe2\x82\xacuro"
        result = inst.doTransform("\xa4uro", None)
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

        result = inst.doTransform(u"\u20acuro", None)
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02doTransfrom(self):
        """ FilenameDescriptor.doTransform passes already encoded names """
        class MyDescriptor(self._acc.FilenameDescriptor):
            def getCharset(this, arg):
                return arg and "us-ascii" or "iso-8859-15"
            def getFileCharset(this, arg):
                return "utf-8"

        inst = MyDescriptor('a', self._priv)
        topass = inst.doTransform("\xa4uro", None)
        inst.doTransform(topass, True)

    def test03doTransform(self):
        """ FilenameDescriptor.doTransform rejects non unicode/string """
        inst = self._acc.FilenameDescriptor('a', self._priv)

        try:
            inst.doTransform(self, "foo")
        except TypeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise TypeError")

_tests.append(FilenameDescriptorTestCase)


class CommandlineDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.CommandlineDescriptor """

    def test00Descriptor(self):
        """ CommandlineDescriptor is a BasePremapDescriptor """
        inst = self._acc.CommandlineDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BasePremapDescriptor),
            "Is not a BasePremapDescriptor"
        )

    def test01doTransform(self):
        """ CommandlineDescriptor.doTransform treats empty input as None """
        inst = self._acc.CommandlineDescriptor('a', self._priv)

        expected = None
        result = inst.doTransform('', "foo")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02doTransform(self):
        """ CommandlineDescriptor.doTransform passes tuples """
        inst = self._acc.CommandlineDescriptor('a', self._priv)

        expected = ('a', 'b')
        result = inst.doTransform(('a', 'b'), "foo")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03doTransform(self):
        """ CommandlineDescriptor.doTransform rejects non unicode/string """
        inst = self._acc.CommandlineDescriptor('a', self._priv)

        try:
            inst.doTransform(self, "foo")
        except TypeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise TypeError")

    def test04doTransform(self):
        """ CommandlineDescriptor.doTransform transforms strings/unicode """
        from svnmailer import util

        tests = (
             'perl\xe2\x82\xac -le "print q(Andr\xc3\xa9)"',
            u'perl\u20ac -le "print q(Andr\xe9)"'
        )
        inst = self._acc.CommandlineDescriptor('a', self._priv)

        expected = ['perl\xa4', u'-le', u'print q(Andr\xe9)']
        if util.filename.unicode_system:
            expected[0] = u'perl\u20ac'

        expected = tuple(expected)
        for test in tests:
            result = inst.doTransform(test,
                {'encoding': "utf-8", 'path_encoding': "iso-8859-15"}
            )
            self.failUnlessEqual(expected, result,
                "Input %r expected %r, got %r" % (test, expected, result)
            )

    def test05doTransform(self):
        """ CommandlineDescriptor.doTransform rejects invalid command lines """
        inst = self._acc.CommandlineDescriptor('a', self._priv)

        try:
            inst.doTransform('"', None)
        except ValueError:
            # cool
            pass
        else:
            self.fail("Doesn't raise ValueError")

    def test06doTransform(self):
        """ CommandlineDescriptor.doTransform raises UnicodeError """
        inst = self._acc.CommandlineDescriptor('a', self._priv)

        from svnmailer import util
        test = util.filename.unicode_system and "Andr\xe9" or u"Andr\xe9"

        try:
            inst.doTransform(test,
                {'encoding': None, 'path_encoding': 'us-ascii'}
            )
        except UnicodeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise UnicodeError")

_tests.append(CommandlineDescriptorTestCase)


class QuotedStringTestCase(accessorTestBase):
    """ svnmailer.struct_accessors._QuotedString """

    def test00subclass(self):
        """ _QuotedString is a str """
        inst = self._acc._QuotedString()
        self.failUnless(
            isinstance(inst, str),
            "Is not a str"
        )

    def test01attribute(self):
        """ _QuotedString has _parsed_quoted_string attribute """
        inst = self._acc._QuotedString()
        self.failUnless(
            hasattr(inst, '_parsed_quoted_string'),
            "Has no '_parsed_quoted_string' attribute"
        )

    def test02new(self):
        """ _QuotedString rejects invalid input """
        tests = (r' bar', r'"\"bar', r'"abc\\" ', r'"woo " ')

        for test in tests:
            try:
                self._acc._QuotedString(test)
            except ValueError:
                # cool
                pass
            else:
                self.fail("Input %r doesn't raise ValueError" % test)

    def test03new(self):
        """ _QuotedString unquotes strings properly """
        tests = (
            ('',         r''),
            ('',         r'""'),
            ('bar',      r'bar'),
            ('\\"bar',   r'\"bar'),
            ('abc\\',    r'"abc\\"'),
            ('woo "!\\', r'"woo \"!\\"'),
        )
        for expected, test in tests:
            result = self._acc._QuotedString(test)
            self.failUnlessEqual(expected, result,
                "Input %r expected %r, got %r" % (test, expected, str(result))
            )

    def test04repr(self):
        """ _QuotedString represents itself properly """
        inst = self._acc._QuotedString(r'"woo \"!\\"')

        expected = repr(r'"woo \"!\\"')
        result = repr(inst)
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(QuotedStringTestCase)


class QuotedstringDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.QuotedstringDescriptor """

    def test00Descriptor(self):
        """ QuotedstringDescriptor is a BasePremapDescriptor """
        inst = self._acc.QuotedstringDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BasePremapDescriptor),
            "Is not a BasePremapDescriptor"
        )

    def test01doTransform(self):
        """ QuotedstringDescriptor.doTransform passes quoted strings """
        inst = self._acc.QuotedstringDescriptor('a', self._priv)

        expected = self._acc._QuotedString('b')
        result = inst.doTransform(expected, "foo")
        self.failUnless(expected is result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02doTransform(self):
        """ QuotedstringDescriptor.doTransform rejects non string types """
        inst = self._acc.QuotedstringDescriptor('a', self._priv)

        try:
            inst.doTransform(u"foo", "bar")
        except TypeError:
            # cool
            pass
        else:
            self.fail("Doesn't raise a TypeError")

    def test03doTransform(self):
        """ QuotedstringDescriptor.doTransform returns _QuotedString types """
        inst = self._acc.QuotedstringDescriptor('a', self._priv)

        expected = self._acc._QuotedString
        result = inst.doTransform("foo", "bar")
        self.failUnless(
            isinstance(result, expected),
            "Is not of type %r, but %r" % (expected, result)
        )

_tests.append(QuotedstringDescriptorTestCase)


class StdinDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.StdinDescriptor """

    def test00Descriptor(self):
        """ StdinDescriptor is a BaseDescriptor """
        inst = self._acc.StdinDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BaseDescriptor),
            "Is not a BaseDescriptor"
        )

    def test01substitute(self):
        """ StdinDescriptor.substitute returns stdin as string"""
        import cStringIO, sys
        inst = self._acc.StdinDescriptor('a', self._priv)

        expected = "foooo"
        fp = cStringIO.StringIO()
        fp.write("foooo")
        fp.seek(0)
        fp2 = cStringIO.StringIO()
        fp2.write("baaaar")
        fp2.seek(0)

        old_fp = sys.stdin
        try:
            sys.stdin = fp
            result = inst.substitute(None, None, None)
            sys.stdin = fp2
            result = inst.substitute(None, None, None)
        finally:
            sys.stdin = old_fp

        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(StdinDescriptorTestCase)


class MailActionTestCase(accessorTestBase):
    """ svnmailer.struct_accessors._MailAction """

    def test00init(self):
        """ _MailAction rejects string with less than 2 tokens """
        tests = (" ", " a  \t", "10")
        for test in tests:
            try:
                self._acc._MailAction(test)
                self.fail("Input %r doesn't raise ValueError" % (test,))
            except ValueError:
                # fine
                pass

    def test01init(self):
        """ _MailAction rejects non-int size limit """
        tests = ("a split", "0.0 split", "0xa split")
        for test in tests:
            try:
                self._acc._MailAction(test)
                self.fail("Input %r doesn't raise ValueError" % (test,))
            except ValueError:
                # fine
                pass

    def test02init(self):
        """ _MailAction accepts integer size limit """
        expected = 10
        result = self._acc._MailAction("10 split").maxbytes
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03init(self):
        """ _MailAction rejects invalid mode tokens """
        tests = ("10 foo",)
        for test in tests:
            try:
                self._acc._MailAction(test)
                self.fail("Input %r doesn't raise ValueError" % (test,))
            except ValueError:
                # fine
                pass

    def test04init(self):
        """ _MailAction accepts valid mode tokens """
        tests = (
            ([10, "truncate"], "10 Truncate"),
            ([11, "truncate"], "11 truncate"),
            ([12, "split"],    "12 SPLIT"),
            ([13 ,"split"],    "13 split"),
            ([14, "showurls"], "14 showurls"),
            ([15, "showurls"], "15 ShowUrls"),
        )
        for expected, test in tests:
            inst = self._acc._MailAction(test)
            expected = expected + [False, None, ()]
            result = [
                inst.maxbytes, inst.mode, inst.truncate, inst.drop, inst.scope
            ]
            self.failUnlessEqual(expected, result,
                "Input %r expected %r, got %r" % (test, expected, result)
            )

    def test05init(self):
        """ _MailAction rejects invalid /truncate modifiers """
        tests = ("10 truncate/truncate",)
        for test in tests:
            try:
                self._acc._MailAction(test)
                self.fail("Input %r doesn't raise ValueError" % (test,))
            except ValueError:
                # fine
                pass

    def test06init(self):
        """ _MailAction accepts valid /truncate modifiers """
        tests = (
            ([10, "showurls"], "10 showurls/truncate"),
            ([11, "split"],    "11 split/truncate"),
        )
        for expected, test in tests:
            inst = self._acc._MailAction(test)
            expected = expected + [True, None, ()]
            result = [
                inst.maxbytes, inst.mode, inst.truncate, inst.drop, inst.scope
            ]
            self.failUnlessEqual(expected, result,
                "Input %r expected %r, got %r" % (test, expected, result)
            )

    def test07init(self):
        """ _MailAction rejects invalid /<drop> modifiers """
        tests = (
            "10 showurls/11", "12 truncate/13", "14 split/foo",
            "15 showurls/truncate/16", "17 split/truncate/bar",
        )
        for test in tests:
            try:
                self._acc._MailAction(test)
                self.fail("Input %r doesn't raise ValueError" % (test,))
            except ValueError:
                # fine
                pass

    def test08init(self):
        """ _MailAction accepts valid /<drop> modifiers """
        tests = (
            ([10, "split", True, 11],  "10 split/truncate/11"),
            ([12, "split", False, 13], "12 split/13"),
        )
        for expected, test in tests:
            inst = self._acc._MailAction(test)
            expected = expected + [()]
            result = [
                inst.maxbytes, inst.mode, inst.truncate, inst.drop, inst.scope
            ]
            self.failUnlessEqual(expected, result,
                "Input %r expected %r, got %r" % (test, expected, result)
            )

    def test09init(self):
        """ _MailAction rejects too much modifier tokens """
        tests = (
            "10 split/11/12", "13 split/14/foo",
            "15 split/truncate/16/17", "18 split/truncate/19/bar",
        )
        for test in tests:
            try:
                self._acc._MailAction(test)
                self.fail("Input %r doesn't raise ValueError" % (test,))
            except ValueError:
                # fine
                pass

    def test10init(self):
        """ _MailAction rejects invalid scope tokens """
        tests = ("10 split foo", "11 split locks foo")
        for test in tests:
            try:
                self._acc._MailAction(test)
                self.fail("Input %r doesn't raise ValueError" % (test,))
            except ValueError:
                # fine
                pass

    def test11init(self):
        """ _MailAction accepts valid scope tokens """
        tests = (
            ([10, "split", True, 11, ("revprop-changes",)],
                "10 split/truncate/11 revprop-changes"),
            ([12, "split", False, 13, ("locks",)], "12 split/13 locks"),
            ([14, "split", True, 15, ("revprop-changes", "locks")],
                "14 split/truncate/15 locks revprop-changes"),
        )
        for expected, test in tests:
            inst = self._acc._MailAction(test)
            expected[4] = dict.fromkeys(expected[4])
            result = [
                inst.maxbytes, inst.mode, inst.truncate, inst.drop,
                dict.fromkeys(inst.scope)
            ]
            self.failUnlessEqual(expected, result,
                "Input %r expected %r, got %r" % (test, expected, result)
            )

    def test12repr(self):
        """ _MailAction.__repr__ returns the proper string """
        tests = (
            ("1 truncate",          "\t1  TRUNCATE"),
            ("2 showurls/truncate", "2 ShowUrls/Truncate"),
            ("3 split/truncate/4",  "3 SPLIT/Truncate/4"),
            ("5 split/6",           "5 Split/6"),
            ("7 truncate locks revprop-changes",
                "7 Truncate Locks Revprop-Changes"),
            ("8 split/truncate/9 locks", "8 split/TrunCate/9 LOCKS"),
        )
        for expected, test in tests:
            result = repr(self._acc._MailAction(test))
            self.failUnlessEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )

_tests.append(MailActionTestCase)


class MailactionDescriptorTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.MailactionDescriptor """

    def test00Descriptor(self):
        """ MailactionDescriptor is a BasePremapDescriptor """
        inst = self._acc.MailactionDescriptor('a', self._priv)
        self.failUnless(
            isinstance(inst, self._acc.BasePremapDescriptor),
            "Is not a BasePremapDescriptor"
        )

    def test01doTransform(self):
        """ MailactionDescriptor.doTransform passes mail actions """
        inst = self._acc.MailactionDescriptor('a', self._priv)

        expected = self._acc._MailAction('10 split')
        result = inst.doTransform(expected, "foo")
        self.failUnless(expected is result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02doTransform(self):
        """ MailactionDescriptor.doTransform rejects non string types """
        inst = self._acc.QuotedstringDescriptor('a', self._priv)

        try:
            inst.doTransform(u"foo", "bar")
            self.fail("Doesn't raise a TypeError")
        except TypeError:
            # cool
            pass

    def test03doTransform(self):
        """ MailactionDescriptor.doTransform returns _MailAction types """
        inst = self._acc.MailactionDescriptor('a', self._priv)

        expected = self._acc._MailAction
        result = inst.doTransform("10 split", "bar")
        self.failUnless(
            isinstance(result, expected),
            "Is not of type %r, but %r" % (expected, result)
        )

    def test04doTransform(self):
        """ MailactionDescriptor.doTransform turns empty value to None """
        inst = self._acc.MailactionDescriptor('a', self._priv)

        expected = None
        result = inst.doTransform("", "foo")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )
 
_tests.append(MailactionDescriptorTestCase)


class typemapTestCase(accessorTestBase):
    """ svnmailer.struct_accessors.typemap """

    def test00keys(self):
        """ typemap contains all specified keys and nothing more """
        keys = dict.fromkeys([
            'unicode', 'string', 'int', 'bool', 'humanbool', 'regex',
            'token', 'tokenlist', 'filename', 'unicommand', 'quotedstr',
            'stdin', 'mailaction',
        ])
        for key in self._acc.typemap.keys():
            self.failUnless(keys.has_key(key),
                "Key %r shouldn't exist in typemap" % (key,)
            )
            del keys[key]

        if keys:
            self.fail(
                "The keys %r are missing in typemap" % (tuple(keys.keys()),)
            )

    def test01values(self):
        """ typemap keys point to BaseDescriptors """
        for key, value in self._acc.typemap.items():
            self.failUnless(
                issubclass(value, self._acc.BaseDescriptor),
                "Key %r doesn't point to a BaseDescriptor, but %r" %
                (key, value)
            )

    def test02values(self):
        """ typemap values are all different """
        tmap = self._acc.typemap
        for key, value in tmap.items():
            result = tmap.values().count(value)
            self.failUnlessEqual(1, result,
                "Key %r points to %r, counted %d" % (key, value, result)
            )

    def test03dict(self):
        """ typemap is a read-only dictionary """
        from svnmailer import util
        self.failUnless(
            isinstance(self._acc.typemap, util.ReadOnlyDict),
            "Is not an instance of util.ReadOnlyDict"
        )

_tests.append(typemapTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return unittest.TestSuite([
        unittest.makeSuite(test) for test in _tests
    ])

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity = 2).run(getSuite())
